package rocks.zipcode.atm;

import rocks.zipcode.atm.bank.AccountData;
import rocks.zipcode.atm.bank.Bank;

import java.awt.event.ActionEvent;
import java.util.function.Consumer;
import java.util.function.Supplier;

import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.control.PasswordField;
import javafx.scene.control.TextField;

/**
 * @author ZipCodeWilmington
 */
public class CashMachine {

    @FXML
    private Label title;

    @FXML
    private TextField username;

    @FXML
    private PasswordField password;

    @FXML
    private Label loginFailed;

    private final Bank bank;
    private AccountData accountData = null;
    /*private String message = null;
    private Integer amount = 0;
*/
    public CashMachine(Bank bank) {
        this.bank = bank;
    }

    private Consumer<AccountData> update = data -> {
        accountData = data;
    };

    /*private Consumer<String> update2 = data -> {
        message = data;
    };
    private Consumer<Integer> update3 = data -> {
        amount = data;
    };*/

    public void login(ActionEvent event){

        if(username.getText().equals("user") && password.getText().equals("password")){
            title.setText("Login Success");
        }
        else
            loginFailed.setVisible(true);

    }

    public void login(int id) {
        tryCall(
                () -> bank.getAccountById(id),
                update
        );
    }

   /* public void checkAccount(){
        if(accountData != null){
            tryCall(
                    () -> bank.getAccountType(accountData), update
            );
        }
    }*/

    public void checkBalance() {
        if(accountData != null){
            tryCall(
                    () -> bank.getBalance(accountData), update
            );
        }

    }

    public void deposit(int amount) {
        if (accountData != null) {
            tryCall(
                    () -> bank.deposit(accountData, amount),
                    update
            );
        }
    }

    public void withdraw(int amount) {
        if (accountData != null) {
            tryCall(
                    () -> bank.withdraw(accountData, amount),
                    update
            );
        }
    }

    public void exit() {
        if (accountData != null) {
            accountData = null;
        }
    }

    public String getUser(){
        return "Welcome " + accountData.getName();
    }

    @Override
    public String toString() {
        return accountData != null ? accountData.toString() : "Try giving a valid account number and click submit.";
    }

    private <T> void tryCall(Supplier<ActionResult<T> > action, Consumer<T> postAction) {
        try {
            ActionResult<T> result = action.get();
            if (result.isSuccess()) {
                T data = result.getData();
                postAction.accept(data);
            } else {
                String errorMessage = result.getErrorMessage();
                throw new RuntimeException(errorMessage);
            }
        } catch (Exception e) {
            System.out.println("Error: " + e.getMessage());
        }
    }


    public String goodBye() {
        return "Good Bye!! Thanks for using our Cash Machine";
    }

    public String getBalance() {
        return "The Balance in your account is "+ this.accountData.getBalance();
    }
}
